@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2020 Matthias Geier
+# Copyright (c) 2015-2021 Matthias Geier
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 https://nbsphinx.readthedocs.io/
 
 """
-__version__ = '0.8.0'
+__version__ = '0.8.7'
 
 import collections.abc
 import copy
@@ -43,6 +43,7 @@ import nbconvert
 import nbformat
 import sphinx
 import sphinx.directives
+import sphinx.directives.other
 import sphinx.environment
 import sphinx.errors
 import sphinx.transforms.post_transforms.images
@@ -240,9 +241,9 @@ RST_TEMPLATE = """
 
 {% block markdowncell %}
 {%- if 'nbsphinx-gallery' in cell.metadata
-    or 'nbsphinx-gallery' in cell.metadata.tags
+    or 'nbsphinx-gallery' in cell.metadata.get('tags', [])
     or 'nbsphinx-toctree' in cell.metadata
-    or 'nbsphinx-toctree' in cell.metadata.tags %}
+    or 'nbsphinx-toctree' in cell.metadata.get('tags', []) %}
 {{ cell | extract_gallery_or_toctree }}
 {%- else %}
 {{ cell | save_attachments or super() | replace_attachments }}
@@ -278,7 +279,7 @@ RST_TEMPLATE = """
 
 {% block footer %}
 
-{% if 'application/vnd.jupyter.widget-state+json' in nb.metadata.widgets %}
+{% if 'application/vnd.jupyter.widget-state+json' in nb.metadata.get('widgets', {})%}
 
 .. raw:: html
 
@@ -765,7 +766,7 @@ class Exporter(nbconvert.RSTExporter):
                 # For now, this works only if there is a single MIME bundle
                 (mime_type, data), = bundle.items()
                 text = re.sub(
-                    r'^(\s*\.\. (\|[^|]*\| image|figure)::) attachment:{0}$'
+                    r'^(\s*\.\. ((\|[^|]*\| )?image|figure)::) attachment:{0}$'
                         .format(filename),
                     r'\1 data:{0};base64,{1}'.format(mime_type, data),
                     text, flags=re.MULTILINE)
@@ -938,6 +939,7 @@ class NotebookParser(rst.Parser):
             CreateNotebookSectionAnchors,
             ReplaceAlertDivs,
             CopyLinkedFiles,
+            ForceEquations,
         ]
 
     def parse(self, inputstring, document):
@@ -1053,6 +1055,7 @@ class NotebookParser(rst.Parser):
             prolog = exporter.environment.from_string(
                 env.config.nbsphinx_prolog).render(env=env)
             rst.Parser.parse(self, prolog, document)
+        rst.Parser.parse(self, '.. highlight:: none', document)
         rst.Parser.parse(self, rststring, document)
         if env.config.nbsphinx_epilog:
             epilog = exporter.environment.from_string(
@@ -1226,14 +1229,14 @@ class NbGallery(sphinx.directives.other.TocTree):
         """Wrap GalleryToc arount toctree."""
         ret = super().run()
         try:
-            toctree_wrapper, = ret
+            toctree_wrapper = ret[-1]
             toctree, = toctree_wrapper
-        except ValueError:
+        except (IndexError, TypeError, ValueError):
             return ret
         if not isinstance(toctree, sphinx.addnodes.toctree):
             return ret
         gallerytoc = GalleryToc()
-        gallerytoc += toctree_wrapper
+        gallerytoc.extend(ret)
         return [gallerytoc]
 
 
@@ -1266,6 +1269,9 @@ class CitationParser(html.parser.HTMLParser):
             if name == 'data-cite':
                 self.cite = ':cite:`' + value + '`'
                 return True
+            elif name == 'data-footcite':
+                self.cite = ':footcite:`' + value + '`'
+                return True
         return False
 
     def reset(self):
@@ -1792,6 +1798,21 @@ class CopyLinkedFiles(docutils.transforms.Transform):
             env.nbsphinx_files.setdefault(env.docname, []).append(file)
 
 
+class ForceEquations(docutils.transforms.Transform):
+    """Unconditionally enable equations on notebooks.
+
+    Except if ``nbsphinx_assume_equations`` is set to ``False``.
+
+    """
+
+    default_priority = 900  # after checking for equations in MathDomain
+
+    def apply(self):
+        env = self.document.settings.env
+        if env.config.nbsphinx_assume_equations:
+            env.get_domain('math').data['has_equations'][env.docname] = True
+
+
 class GetSizeFromImages(
         sphinx.transforms.post_transforms.images.BaseImageConverter):
     """Get size from images and store it as node attributes.
@@ -1843,6 +1864,8 @@ def patched_toctree_resolve(self, docname, builder, toctree, *args, **kwargs):
 
 
 def config_inited(app, config):
+    if '.ipynb' not in config.source_suffix:
+        app.add_source_suffix('.ipynb', 'jupyter_notebook')
     for suffix in config.nbsphinx_custom_formats:
         app.add_source_suffix(suffix, 'jupyter_notebook')
 
@@ -1853,6 +1876,65 @@ def config_inited(app, config):
     # see https://github.com/spatialaudio/nbsphinx/issues/409
     app.connect('builder-inited', load_requirejs)
 
+    # http://docs.mathjax.org/en/v3.1-latest/options/document.html
+    # http://docs.mathjax.org/en/v2.7-latest/options/preprocessors/tex2jax.html
+    mathjax_inline_math = [['$', '$'], ['\\(', '\\)']]
+    mathjax_ignore_class = (
+        'tex2jax_ignore'  # MathJax 2 default
+        '|'
+        'mathjax_ignore'  # Mathjax 3 default
+        '|'
+        'document'  # Main page content
+    )
+    mathjax_process_class = (
+        'tex2jax_process'  # MathJax 2 default
+        '|'
+        'mathjax_process'  # Mathjax 3 default
+        '|'
+        'math'  # Used by Sphinx
+        '|'
+        'output_area'  # Jupyter code cells
+    )
+
+    # See also https://github.com/sphinx-doc/sphinx/pull/5504
+    if hasattr(config, 'mathjax3_config') and config.mathjax2_config is None:
+        # NB: If mathjax_path is used in Sphinx >= 4 to load MathJax v2,
+        # this only works if mathjax_config or mathjax2_config is specified.
+        if config.mathjax3_config is None:
+            config.mathjax3_config = {}
+        mathjax3_config = config.mathjax3_config
+        tex = {
+            'inlineMath': mathjax_inline_math,
+            'processEscapes': True,
+        }
+        tex.update(mathjax3_config.get('tex', {}))
+        mathjax3_config['tex'] = tex
+        options = {
+            'ignoreHtmlClass': mathjax_ignore_class,
+            'processHtmlClass': mathjax_process_class,
+        }
+        options.update(mathjax3_config.get('options', {}))
+        mathjax3_config['options'] = options
+    else:
+        if hasattr(config, 'mathjax2_config'):
+            # Sphinx >= 4.0
+            if config.mathjax2_config is None:
+                config.mathjax2_config = {}
+            mathjax2_config = config.mathjax2_config
+        else:
+            # Sphinx < 4.0
+            if config.mathjax_config is None:
+                config.mathjax_config = {}
+            mathjax2_config = config.mathjax_config
+        tex2jax = {
+            'inlineMath': mathjax_inline_math,
+            'processEscapes': True,
+            'ignoreClass': mathjax_ignore_class,
+            'processClass': mathjax_process_class,
+        }
+        tex2jax.update(mathjax2_config.get('tex2jax', {}))
+        mathjax2_config['tex2jax'] = tex2jax
+
 
 def load_requirejs(app):
     config = app.config
@@ -1872,6 +1954,7 @@ def load_requirejs(app):
 
 def builder_inited(app):
     env = app.env
+    env.settings['line_length_limit'] = 100_000_000
     env.nbsphinx_notebooks = {}
     env.nbsphinx_files = {}
     if not hasattr(env, 'nbsphinx_thumbnails'):
@@ -2087,11 +2170,21 @@ def depart_codearea_latex(self, node):
     if lines[0].startswith(r'\fvset{'):  # Sphinx >= 1.6.6 and < 1.8.3
         out.append(lines[0])
         del lines[0]
-    assert 'Verbatim' in lines[0]
-    out.append(lines[0])
+    # Sphinx 4.1.0 added "sphinxuseclass" environments around "sphinxVerbatim"
+    for begin_verbatim, line in enumerate(lines):
+        if line.startswith(r'\begin{sphinxVerbatim}'):
+            break
+    else:
+        assert False
+    for end_verbatim, line in enumerate(reversed(lines)):
+        if line == r'\end{sphinxVerbatim}':
+            break
+    else:
+        assert False
+    out.extend(lines[:begin_verbatim + 1])
     code_lines = (
         [''] * node.get('empty-lines-before', 0) +
-        lines[1:-1] +
+        lines[begin_verbatim + 1:-end_verbatim - 1] +
         [''] * node.get('empty-lines-after', 0)
     )
     prompt = node['prompt']
@@ -2102,8 +2195,7 @@ def depart_codearea_latex(self, node):
         assert code_lines
         code_lines[0] = prefix + code_lines[0]
     out.extend(code_lines)
-    assert 'Verbatim' in lines[-1]
-    out.append(lines[-1])
+    out.extend(lines[-end_verbatim - 1:])
     out.append('}')  # End of scope for colors
     out.append('')
     self.body.append('\n'.join(out))
@@ -2189,7 +2281,6 @@ def do_nothing(self, node):
 
 def setup(app):
     """Initialize Sphinx extension."""
-    app.add_source_suffix('.ipynb', 'jupyter_notebook')
     app.add_source_parser(NotebookParser)
 
     app.add_config_value('nbsphinx_execute', 'auto', rebuild='env')
@@ -2213,6 +2304,7 @@ def setup(app):
     app.add_config_value('nbsphinx_widgets_path', None, rebuild='html')
     app.add_config_value('nbsphinx_widgets_options', {}, rebuild='html')
     app.add_config_value('nbsphinx_thumbnails', {}, rebuild='html')
+    app.add_config_value('nbsphinx_assume_equations', True, rebuild='env')
 
     app.add_directive('nbinput', NbInput)
     app.add_directive('nboutput', NbOutput)
@@ -2249,18 +2341,6 @@ def setup(app):
     # see https://github.com/sphinx-doc/sphinx/issues/2155:
     rst.directives.register_directive('code', sphinx.directives.code.CodeBlock)
 
-    # Work-around until https://github.com/sphinx-doc/sphinx/pull/5504 is done:
-    mathjax_config = app.config._raw_config.setdefault('mathjax_config', {})
-    mathjax_config.setdefault(
-        'tex2jax',
-        {
-            'inlineMath': [['$', '$'], ['\\(', '\\)']],
-            'processEscapes': True,
-            'ignoreClass': 'document',
-            'processClass': 'math|output_area',
-        }
-    )
-
     # Add LaTeX definitions to preamble
     latex_elements = app.config._raw_config.setdefault('latex_elements', {})
     latex_elements['preamble'] = '\n'.join([
